@@ -1,7 +1,7 @@
 
 /*!
  * Stylus - errors
- * Copyright(c) 2010 LearnBoost <dev@learnboost.com>
+ * Copyright (c) Automattic <developer.wordpress.com>
  * MIT Licensed
  */
 
@@ -12,12 +12,6 @@
 exports.ParseError = ParseError;
 exports.SyntaxError = SyntaxError;
 
-/**
- * Inherit from `Error.prototype`.
- */
-
-SyntaxError.prototype.__proto__ = Error.prototype;
-
 /**
  * Initialize a new `ParseError` with the given `msg`.
  *
@@ -28,7 +22,9 @@ SyntaxError.prototype.__proto__ = Error.prototype;
 function ParseError(msg) {
   this.name = 'ParseError';
   this.message = msg;
-  Error.captureStackTrace(this, ParseError);
+  if (Error.captureStackTrace) {
+    Error.captureStackTrace(this, ParseError);
+  }
 }
 
 /**
@@ -47,7 +43,9 @@ ParseError.prototype.__proto__ = Error.prototype;
 function SyntaxError(msg) {
   this.name = 'SyntaxError';
   this.message = msg;
-  Error.captureStackTrace(this, ParseError);
+  if (Error.captureStackTrace) {
+    Error.captureStackTrace(this, ParseError);
+  }
 }
 
 /**
@@ -55,4 +53,3 @@ function SyntaxError(msg) {
  */
 
 SyntaxError.prototype.__proto__ = Error.prototype;
-
