@@ -10,7 +10,7 @@ var utils = require('../utils');
  * @api private
  */
 
-module.exports = function adjust(color, prop, amount){
+function adjust(color, prop, amount){
   utils.assertColor(color, 'color');
   utils.assertString(prop, 'prop');
   utils.assertType(amount, 'unit', 'amount');
@@ -26,3 +26,5 @@ module.exports = function adjust(color, prop, amount){
   hsl[prop] += val;
   return hsl.rgba;
 };
+adjust.params = ['color', 'prop', 'amount'];
+module.exports = adjust;
