@@ -22,7 +22,7 @@ var nodes = require('../nodes')
  * @api public
  */
 
-module.exports = function alpha(color, value){
+function alpha(color, value){
   color = color.rgba;
   if (value) {
     return rgba(
@@ -34,3 +34,5 @@ module.exports = function alpha(color, value){
   }
   return new nodes.Unit(color.a, '');
 };
+alpha.params = ['color', 'value'];
+module.exports = alpha;
