@@ -11,8 +11,7 @@
  */
 
 var utils = require('../utils')
-  , nodes = require('../nodes')
-  , Buffer = require('safer-buffer').Buffer
+  , Buffer = require('buffer').Buffer
   , fs = require('fs')
   , path = require('path')
   , sax = require('sax');
