@@ -14,6 +14,8 @@ var utils = require('../utils');
       var nodes = utils.unwrap(expr).nodes;
       if (1 == nodes.length && 'object' == nodes[0].nodeName) {
         return nodes[0].length;
+      } else if (1 == nodes.length && 'string' == nodes[0].nodeName) {
+        return nodes[0].val.length;
       } else {
         return nodes.length;
       }
