@@ -8,6 +8,8 @@ var nodes = require('../nodes');
  * @api private
  */
 
-module.exports = function math(prop){
+function math(prop){
   return new nodes.Unit(Math[prop.string]);
-};
+}
+math.params = ['prop'];
+module.exports = math;
