@@ -1,6 +1,6 @@
 /*!
  * Stylus - Parser
- * Copyright(c) 2010 LearnBoost <dev@learnboost.com>
+ * Copyright (c) Automattic <developer.wordpress.com>
  * MIT Licensed
  */
 
@@ -51,10 +51,12 @@ var selectorTokens = [
   , '='
   , ':'
   , '&'
+  , '&&'
   , '~'
   , '{'
   , '}'
   , '.'
+  , '..'
   , '/'
 ];
 
@@ -64,8 +66,11 @@ var selectorTokens = [
  */
 
 var pseudoSelectors = [
+  // https://www.w3.org/TR/selectors-4/#logical-combination
   // Logical Combinations
-    'matches'
+    'is'
+  , 'has'
+  , 'where'
   , 'not'
 
   // Linguistic Pseudo-classes
@@ -489,6 +494,7 @@ Parser.prototype = {
 
   looksLikeSelector: function(fromProperty) {
     var i = 1
+      , node
       , brace;
 
     // Real property
@@ -499,7 +505,8 @@ Parser.prototype = {
     // Assume selector when an ident is
     // followed by a selector
     while ('ident' == this.lookahead(i).type
-      && 'newline' == this.lookahead(i + 1).type) i += 2;
+      && ('newline' == this.lookahead(i + 1).type
+         || ',' == this.lookahead(i + 1).type)) i += 2;
 
     while (this.isSelectorToken(i)
       || ',' == this.lookahead(i).type) {
@@ -507,16 +514,89 @@ Parser.prototype = {
       if ('selector' == this.lookahead(i).type)
         return true;
 
-      if ('.' == this.lookahead(i).type && 'ident' == this.lookahead(i + 1).type)
+      if ('&' == this.lookahead(i + 1).type)
         return true;
 
+      // Hash values inside properties
+      if (
+        i > 1 &&
+        'ident' === this.lookahead(i - 1).type &&
+        '.' === this.lookahead(i).type &&
+        'ident' === this.lookahead(i + 1).type
+      ) {
+        while ((node = this.lookahead(i + 2))) {
+          if ([
+            'indent',
+            'outdent',
+            '{',
+            ';',
+            'eos',
+            'selector',
+            'media',
+            'if',
+            'atrule',
+            ')',
+            '}',
+            'unit',
+            '[',
+            'for',
+            'function'
+          ].indexOf(node.type) !== -1) {
+            if (node.type === '[') {
+              while ((node = this.lookahead(i + 3)) && node.type !== ']') {
+                if (~['.', 'unit'].indexOf(node.type)) {
+                  return false;
+                }
+                i += 1
+              }
+            } else {
+              if (this.isPseudoSelector(i + 2)) {
+                return true;
+              }
+
+              if (node.type === ')' && this.lookahead(i + 3) && this.lookahead(i + 3).type === '}') {
+                break;
+              }
+
+              return [
+                'outdent',
+                ';',
+                'eos',
+                'media',
+                'if',
+                'atrule',
+                ')',
+                '}',
+                'unit',
+                'for',
+                'function'
+              ].indexOf(node.type) === -1;
+            }
+          }
+
+          i += 1
+        }
+
+        return true;
+      }
+
+      if ('.' == this.lookahead(i).type && 'ident' == this.lookahead(i + 1).type) {
+        return true;
+      }
+
       if ('*' == this.lookahead(i).type && 'newline' == this.lookahead(i + 1).type)
         return true;
 
+      // Pseudo-elements
       if (':' == this.lookahead(i).type
         && ':' == this.lookahead(i + 1).type)
         return true;
 
+      // #a after an ident and newline
+      if ('color' == this.lookahead(i).type
+        && 'newline' == this.lookahead(i - 1).type)
+        return true;
+
       if (this.looksLikeAttributeSelector(i))
         return true;
 
@@ -524,6 +604,7 @@ Parser.prototype = {
         && '{' == this.lookahead(i + 1).type)
         return false;
 
+      // Hash values inside properties
       if (':' == this.lookahead(i).type
         && !this.isPseudoSelector(i + 1)
         && this.lineContains('.'))
@@ -535,7 +616,7 @@ Parser.prototype = {
       else if ('}' == this.lookahead(i).type) brace = false;
       if (brace && ':' == this.lookahead(i).type) return true;
 
-      // '}' preceded by a space is considered a selector.
+      // '{' preceded by a space is considered a selector.
       // for example "foo{bar}{baz}" may be a property,
       // however "foo{bar} {baz}" is a selector
       if ('space' == this.lookahead(i).type
@@ -550,6 +631,12 @@ Parser.prototype = {
         && this.isPseudoSelector(i))
         return true;
 
+      // Trailing space
+      if ('space' == this.lookahead(i).type
+        && 'newline' == this.lookahead(i + 1).type
+        && '{' == this.lookahead(i + 2).type)
+        return true;
+
       if (',' == this.lookahead(i).type
         && 'newline' == this.lookahead(i + 1).type)
         return true;
@@ -635,7 +722,6 @@ Parser.prototype = {
       case 'selector':
       case 'conditional':
       case 'function':
-      case 'media':
       case 'atrule':
       case 'for':
         return true;
@@ -701,7 +787,7 @@ Parser.prototype = {
               if (this.accept(',')) key = this.id().name;
               this.expect('in');
               var each = new nodes.Each(val, key, this.expression());
-              block = new nodes.Block;
+              block = new nodes.Block(this.parent, each);
               block.push(stmt);
               each.block = block;
               stmt = each;
@@ -734,8 +820,8 @@ Parser.prototype = {
    */
 
   stmt: function() {
-    var type = this.peek().type;
-    switch (type) {
+    var tok = this.peek(), selector;
+    switch (tok.type) {
       case 'keyframes':
         return this.keyframes();
       case '-moz-document':
@@ -752,11 +838,12 @@ Parser.prototype = {
       case 'atrule':
       case 'ident':
       case 'scope':
+      case 'supports':
       case 'unless':
       case 'function':
       case 'for':
       case 'if':
-        return this[type]();
+        return this[tok.type]();
       case 'return':
         return this.return();
       case '{':
@@ -764,17 +851,25 @@ Parser.prototype = {
       default:
         // Contextual selectors
         if (this.stateAllowsSelector()) {
-          switch (type) {
+          switch (tok.type) {
             case 'color':
             case '~':
             case '>':
             case '<':
             case ':':
             case '&':
+            case '&&':
             case '[':
             case '.':
             case '/':
-              return this.selector();
+              selector = this.selector();
+              selector.column = tok.column;
+              selector.lineno = tok.lineno;
+              return selector;
+            // relative reference
+            case '..':
+              if ('/' == this.lookahead(2).type)
+                return this.selector();
             case '+':
               return 'function' == this.lookahead(2).type
                 ? this.functionCall()
@@ -783,7 +878,12 @@ Parser.prototype = {
               return this.property();
             // keyframe blocks (10%, 20% { ... })
             case 'unit':
-              if (this.looksLikeKeyframe()) return this.selector();
+              if (this.looksLikeKeyframe()) {
+                selector = this.selector();
+                selector.column = tok.column;
+                selector.lineno = tok.lineno;
+                return selector;
+              }
             case '-':
               if ('{' == this.lookahead(2).type)
                 return this.property();
@@ -804,6 +904,7 @@ Parser.prototype = {
   block: function(node, scope) {
     var delim
       , stmt
+      , next
       , block = this.parent = new nodes.Block(this.parent, node);
 
     if (false === scope) block.scope = false;
@@ -828,7 +929,15 @@ Parser.prototype = {
         this.accept(';');
         this.skipWhitespace();
       } else {
-        if (this.accept('newline') || this.accept('indent')) continue;
+        if (this.accept('newline')) continue;
+        // skip useless indents and comments
+        next = this.lookahead(2).type;
+        if ('indent' == this.peek().type
+          && ~['outdent', 'newline', 'comment'].indexOf(next)) {
+          this.skip(['indent', 'outdent']);
+          continue;
+        }
+        if ('eos' == this.peek().type) return block;
         stmt = this.statement();
         this.accept(';');
       }
@@ -911,28 +1020,37 @@ Parser.prototype = {
    */
 
   if: function() {
-    this.expect('if');
+    var token = this.expect('if');
+
     this.state.push('conditional');
     this.cond = true;
     var node = new nodes.If(this.expression())
       , cond
-      , block;
+      , block
+      , item;
+
+    node.column = token.column;
+
     this.cond = false;
     node.block = this.block(node, false);
-    // ignore newlines and comments
-    while ('newline' == this.peek().type
-      || 'comment' == this.peek().type) this.next();
+    this.skip(['newline', 'comment']);
     while (this.accept('else')) {
-      if (this.accept('if')) {
+      token = this.accept('if');
+      if (token) {
         this.cond = true;
         cond = this.expression();
         this.cond = false;
         block = this.block(node, false);
-        node.elses.push(new nodes.If(cond, block));
+        item = new nodes.If(cond, block);
+
+        item.column = token.column;
+
+        node.elses.push(item);
       } else {
         node.elses.push(this.block(node, false));
         break;
       }
+      this.skip(['newline', 'comment']);
     }
     this.state.pop();
     return node;
@@ -954,18 +1072,23 @@ Parser.prototype = {
   },
 
   /**
-   * atrule selector? block
+   * atrule selector? block?
    */
 
   atrule: function(){
     var type = this.expect('atrule').val
-      , node = new nodes.Atrule(type);
+      , node = new nodes.Atrule(type)
+      , tok;
     this.skipSpacesAndComments();
     node.segments = this.selectorParts();
     this.skipSpacesAndComments();
-    this.state.push('atrule');
-    node.block = this.block(node);
-    this.state.pop();
+    tok = this.peek().type;
+    if ('indent' == tok || '{' == tok || ('newline' == tok
+      && '{' == this.lookahead(2).type)) {
+      this.state.push('atrule');
+      node.block = this.block(node);
+      this.state.pop();
+    }
     return node;
   },
 
@@ -982,6 +1105,93 @@ Parser.prototype = {
     return nodes.null;
   },
 
+  /**
+   * supports
+   */
+
+  supports: function(){
+    this.expect('supports');
+    var node = new nodes.Supports(this.supportsCondition());
+    this.state.push('atrule');
+    node.block = this.block(node);
+    this.state.pop();
+    return node;
+  },
+
+  /**
+   *   supports negation
+   * | supports op
+   * | expression
+   */
+
+  supportsCondition: function(){
+    var node = this.supportsNegation()
+      || this.supportsOp();
+    if (!node) {
+      this.cond = true;
+      node = this.expression();
+      this.cond = false;
+    }
+    return node;
+  },
+
+  /**
+   * 'not' supports feature
+   */
+
+  supportsNegation: function(){
+    if (this.accept('not')) {
+      var node = new nodes.Expression;
+      node.push(new nodes.Literal('not'));
+      node.push(this.supportsFeature());
+      return node;
+    }
+  },
+
+  /**
+   * supports feature (('and' | 'or') supports feature)+
+   */
+
+  supportsOp: function(){
+    var feature = this.supportsFeature()
+      , op
+      , expr;
+    if (feature) {
+      expr = new nodes.Expression;
+      expr.push(feature);
+      while (op = this.accept('&&') || this.accept('||')) {
+        expr.push(new nodes.Literal('&&' == op.val ? 'and' : 'or'));
+        expr.push(this.supportsFeature());
+      }
+      return expr;
+    }
+  },
+
+  /**
+   *   ('(' supports condition ')')
+   * | feature
+   */
+
+  supportsFeature: function(){
+    this.skipSpacesAndComments();
+    if ('(' == this.peek().type) {
+      var la = this.lookahead(2).type;
+
+      if ('ident' == la || '{' == la) {
+        return this.feature();
+      } else {
+        this.expect('(');
+        var node = new nodes.Expression;
+        node.push(new nodes.Literal('('));
+        node.push(this.supportsCondition());
+        this.expect(')')
+        node.push(new nodes.Literal(')'));
+        this.skipSpacesAndComments();
+        return node;
+      }
+    }
+  },
+
   /**
    * extend
    */
@@ -989,12 +1199,25 @@ Parser.prototype = {
   extend: function(){
     var tok = this.expect('extend')
       , selectors = []
+      , sel
       , node
       , arr;
 
     do {
       arr = this.selectorParts();
-      if (arr.length) selectors.push(new nodes.Selector(arr));
+
+      if (!arr.length) continue;
+
+      sel = new nodes.Selector(arr);
+      selectors.push(sel);
+
+      if ('!' !== this.peek().type) continue;
+
+      tok = this.lookahead(2);
+      if ('ident' !== tok.type || 'optional' !== tok.val.name) continue;
+
+      this.skip(['!', 'ident']);
+      sel.optional = true;
     } while(this.accept(','));
 
     node = new nodes.Extend(selectors);
@@ -1009,7 +1232,7 @@ Parser.prototype = {
 
   media: function() {
     this.expect('media');
-    this.state.push('media');
+    this.state.push('atrule');
     var media = new nodes.Media(this.queries());
     media.block = this.block(media);
     this.state.pop();
@@ -1034,8 +1257,8 @@ Parser.prototype = {
 
   /**
    *   expression
-   * | (ident | 'not')? ident ('and' queryExpr)*
-   * | queryExpr ('and' queryExpr)*
+   * | (ident | 'not')? ident ('and' feature)*
+   * | feature ('and' feature)*
    */
 
   query: function() {
@@ -1051,7 +1274,7 @@ Parser.prototype = {
       this.cond = true;
       expr = this.expression();
       this.cond = false;
-      query.push(new nodes.QueryExpr(expr.nodes));
+      query.push(new nodes.Feature(expr.nodes));
       return query;
     }
 
@@ -1071,7 +1294,7 @@ Parser.prototype = {
     }
 
     do {
-      query.push(this.queryExpr());
+      query.push(this.feature());
     } while (this.accept('&&'));
 
     return query;
@@ -1081,16 +1304,16 @@ Parser.prototype = {
    * '(' ident ( ':'? expression )? ')'
    */
 
-  queryExpr: function() {
+  feature: function() {
     this.skipSpacesAndComments();
     this.expect('(');
     this.skipSpacesAndComments();
-    var node = new nodes.QueryExpr(this.interpolate());
+    var node = new nodes.Feature(this.interpolate());
     this.skipSpacesAndComments();
     this.accept(':')
     this.skipSpacesAndComments();
     this.inProperty = true;
-    node.expr = this.expression();
+    node.expr = this.list();
     this.inProperty = false;
     this.skipSpacesAndComments();
     this.expect(')');
@@ -1150,7 +1373,7 @@ Parser.prototype = {
   },
 
   /**
-   * namespace ident? string
+   * namespace ident? (string | url)
    */
 
   namespace: function() {
@@ -1164,7 +1387,7 @@ Parser.prototype = {
     }
     this.skipSpacesAndComments();
 
-    str = this.expect('string').val;
+    str = this.accept('string') || this.url();
     this.allowPostfix = true;
     return new nodes.Namespace(str, prefix);
   },
@@ -1178,7 +1401,9 @@ Parser.prototype = {
       , keyframes;
 
     this.skipSpacesAndComments();
-    keyframes = new nodes.Keyframes(this.interpolate(), tok.val);
+    keyframes = new nodes.Keyframes(this.selectorParts(), tok.val);
+    keyframes.column = tok.column;
+
     this.skipSpacesAndComments();
 
     // block
@@ -1261,8 +1486,6 @@ Parser.prototype = {
       case '*':
       case '%':
       case '**':
-      case 'and':
-      case 'or':
       case '&&':
       case '||':
       case '>':
@@ -1287,7 +1510,6 @@ Parser.prototype = {
               return this.property();
             // Part of a selector
             case 'root':
-            case 'media':
             case 'atblock':
             case 'atrule':
               return '[' == la
@@ -1311,7 +1533,6 @@ Parser.prototype = {
           case 'root':
             return this.selector();
           case 'for':
-          case 'media':
           case 'selector':
           case 'function':
           case 'conditional':
@@ -1416,6 +1637,8 @@ Parser.prototype = {
       }
     } while (this.accept(',') || this.accept('newline'));
 
+    if ('selector-parts' == this.currentState()) return group.nodes;
+
     this.state.push('selector');
     group.block = this.block(group);
     this.state.pop();
@@ -1476,9 +1699,11 @@ Parser.prototype = {
    */
 
   assignment: function() {
-    var op
-      , node
-      , name = this.id().name;
+    var
+      op,
+      node,
+      ident = this.id(),
+      name = ident.name;
 
     if (op =
          this.accept('=')
@@ -1493,12 +1718,17 @@ Parser.prototype = {
       // @block support
       if (expr.isEmpty) this.assignAtblock(expr);
       node = new nodes.Ident(name, expr);
+
+      node.lineno = ident.lineno;
+      node.column = ident.column;
+
       this.state.pop();
 
       switch (op.type) {
         case '?=':
           var defined = new nodes.BinOp('is defined', node)
-            , lookup = new nodes.Ident(name);
+            , lookup = new nodes.Expression;
+          lookup.push(new nodes.Ident(name));
           node = new nodes.Ternary(defined, lookup, node);
           break;
         case '+=':
@@ -1573,7 +1803,10 @@ Parser.prototype = {
   functionCall: function() {
     var withBlock = this.accept('+');
     if ('url' == this.peek().val.name) return this.url();
-    var name = this.expect('function').val.name;
+
+    var tok = this.expect('function').val;
+    var name = tok.name;
+
     this.state.push('function arguments');
     this.parens++;
     var args = this.args();
@@ -1581,6 +1814,10 @@ Parser.prototype = {
     this.parens--;
     this.state.pop();
     var call = new nodes.Call(name, args);
+
+    call.column = tok.column;
+    call.lineno = tok.lineno;
+
     if (withBlock) {
       this.state.push('function');
       call.block = this.block(call);
@@ -1594,7 +1831,9 @@ Parser.prototype = {
    */
 
   functionDefinition: function() {
-    var name = this.expect('function').val.name;
+    var
+      tok = this.expect('function'),
+      name = tok.val.name;
 
     // params
     this.state.push('function params');
@@ -1607,6 +1846,10 @@ Parser.prototype = {
     // Body
     this.state.push('function');
     var fn = new nodes.Function(name, params);
+
+    fn.column = tok.column;
+    fn.lineno = tok.lineno;
+
     fn.block = this.block(fn);
     this.state.pop();
     return new nodes.Ident(name, fn);
@@ -1894,7 +2137,9 @@ Parser.prototype = {
       || this.accept('+')
       || this.accept('-')) {
       this.operand = true;
-      node = new nodes.UnaryOp(op.type, this.unary());
+      node = this.unary();
+      if (!node) this.error('illegal unary "' + op + '"');
+      node = new nodes.UnaryOp(op.type, node);
       this.operand = false;
       return node;
     }
@@ -1902,7 +2147,7 @@ Parser.prototype = {
   },
 
   /**
-   *   member ('[' expression ']' ('.' id)? '='?)+
+   *   member ('[' expression ']')+ '='?
    * | member
    */
 
@@ -1912,17 +2157,13 @@ Parser.prototype = {
     while (this.accept('[')) {
       node = new nodes.BinOp('[]', node, this.expression());
       this.expect(']');
-      if (this.accept('.')) {
-        id = new nodes.Ident(this.expect('ident').val.string);
-        node = new nodes.Member(node, id);
-      }
-      // TODO: TernaryOp :)
-      if (this.accept('=')) {
-        node.op += '=';
-        node.val = this.expression();
-        // @block support
-        if (node.val.isEmpty) this.assignAtblock(node.val);
-      }
+    }
+    // TODO: TernaryOp :)
+    if (this.accept('=')) {
+      node.op += '=';
+      node.val = this.list();
+      // @block support
+      if (node.val.isEmpty) this.assignAtblock(node.val);
     }
     return node;
   },
@@ -1931,7 +2172,7 @@ Parser.prototype = {
    *   primary ('.' id)+ '='?
    * | primary
    */
-  
+
   member: function() {
     var node = this.primary();
     if (node) {
@@ -1941,7 +2182,7 @@ Parser.prototype = {
       }
       this.skipSpaces();
       if (this.accept('=')) {
-        node.val = this.expression();
+        node.val = this.list();
         // @block support
         if (node.val.isEmpty) this.assignAtblock(node.val);
       }
@@ -1956,7 +2197,7 @@ Parser.prototype = {
 
   object: function(){
     var obj = new nodes.Object
-      , id, val, comma;
+      , id, val, comma, hash;
     this.expect('{');
     this.skipWhitespace();
 
@@ -1966,12 +2207,21 @@ Parser.prototype = {
 
       if (!comma) this.accept(',');
       id = this.accept('ident') || this.accept('string');
-      if (!id) this.error('expected "ident" or "string", got {peek}');
-      id = id.val.hash;
+
+      if (!id) {
+        this.error('expected "ident" or "string", got {peek}');
+      }
+
+      hash = id.val.hash;
+
       this.skipSpacesAndComments();
       this.expect(':');
+
       val = this.expression();
-      obj.set(id, val);
+
+      obj.setValue(hash, val);
+      obj.setKey(hash, id.val);
+
       comma = this.accept(',');
       this.skipWhitespace();
     }
@@ -1995,7 +2245,7 @@ Parser.prototype = {
 
   primary: function() {
     var tok;
-    this.skipSpacesAndComments();
+    this.skipSpaces();
 
     // Parenthesis
     if (this.accept('(')) {
@@ -2025,6 +2275,7 @@ Parser.prototype = {
       case 'string':
       case 'literal':
       case 'boolean':
+      case 'comment':
         return this.next().val;
       case !this.cond && '{':
         return this.object();
