@@ -9,6 +9,7 @@
  */
 
 var Compiler = require('./compiler')
+  , Buffer = require('buffer').Buffer
   , SourceMapGenerator = require('source-map').SourceMapGenerator
   , basename = require('path').basename
   , extname = require('path').extname
@@ -79,7 +80,7 @@ SourceMapper.prototype.compile = function(){
     map = this.map.toString();
     url = 'data:application/json;'
       + (this.utf8 ?  'charset=utf-8;' : '') + 'base64,'
-      + new Buffer(map).toString('base64');
+      + Buffer.from(map).toString('base64');
   }
   if (this.inline || false !== this.comment)
     css += '/*# sourceMappingURL=' + url + ' */';
